import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:phone_form_field/phone_form_field.dart';
import 'package:phone_form_field/src/widgets/country_selector_navigator.dart';
import 'package:phone_numbers_parser/phone_numbers_parser.dart';

import '../models/countries.dart';
import 'flag_dial_code_chip.dart';

/// Form Field for phone input
///
/// You can customize the phone number type that are valid with
/// the [PhoneNumberType] parameter.
///
/// To customize the look use [decoration], [inputTextStyle] and [cursorColor]
///
/// To customize how the country selection is displayed use the [electorConfig] param.
///
/// The rest of the parameters should be self explanatory or common to regular TextFormField.
class PhoneFormField extends FormField<PhoneNumber> {
  final LightPhoneParser parser;
  final bool autofocus;
  final bool showFlagInInput;

  /// input decoration applied to the input
  final InputDecoration decoration;
  final TextStyle inputTextStyle;
  final Color? cursorColor;
  final bool withHint;
  final ValueChanged<PhoneNumber?>? onChanged;

  /// configures the way the country picker selector is shown
  final CountrySelectorNavigator selectorNavigator;

  static String? Function(PhoneNumber?) _getDefaultValidator(
    PhoneNumberType? type,
    AutovalidateMode autovalidateMode,
  ) {
    return (PhoneNumber? phoneNumber) {
      if (phoneNumber != null && phoneNumber.validate(type)) return null;
      if (phoneNumber?.nsn == '' &&
          autovalidateMode == AutovalidateMode.onUserInteraction) return null;
      return 'Invalid phone number';
    };
  }

  PhoneFormField({
    // form field params
    Key? key,
    PhoneNumber? initialValue,
    bool enabled = true,
    AutovalidateMode autovalidateMode = AutovalidateMode.onUserInteraction,
    void Function(PhoneNumber)? onSaved,
    // our params
    this.onChanged,
    bool lightParser = true,
    this.autofocus = true,
    this.showFlagInInput = true,
    this.decoration = const InputDecoration(border: UnderlineInputBorder()),
    this.inputTextStyle = const TextStyle(),
    this.cursorColor,
    this.withHint = true,
    PhoneNumberType? phoneNumberType,
    @Deprecated('use selectorNavigator instead') SelectorConfig? selectorConfig,
    CountrySelectorNavigator? selectorNavigator = const BottomSheetNavigator(),
  })  : parser = lightParser ? LightPhoneParser() : PhoneParser(),
        // for backward compatibility
        selectorNavigator = selectorConfig != null && selectorNavigator == null
            ? CountrySelectorNavigator.fromSelectorConfig(selectorConfig)
            : selectorNavigator,
        // end of backward compatibility
        super(
          key: key,
          initialValue: initialValue,
          onSaved: (p) => onSaved != null ? onSaved(p!) : (p) {},
          enabled: enabled,
          autovalidateMode: autovalidateMode,
          validator: _getDefaultValidator(phoneNumberType, autovalidateMode),
          builder: (field) {
            final state = field as _PhoneFormFieldState;
            return state.builder();
          },
        );

  @override
  _PhoneFormFieldState createState() => _PhoneFormFieldState();
}

class _PhoneFormFieldState extends FormFieldState<PhoneNumber> {
  final FocusNode _focusNode = FocusNode();
  late final TextEditingController _controller;

  _PhoneFormFieldState();

  String get isoCode => value?.isoCode ?? 'US';

  get isOutlineBorder => widget.decoration.border is OutlineInputBorder;

  @override
  PhoneFormField get widget => super.widget as PhoneFormField;

  @override
  void initState() {
    super.initState();
    _controller = TextEditingController(text: value?.nsn);
    _controller.addListener(_onNationalNumberChanges);
    _focusNode.addListener(() => setState(() {}));
  }

  @override
  void didChange(PhoneNumber? value) {
    super.didChange(value);
    if (widget.onChanged != null) {
      widget.onChanged!(value);
    }
  }

  @override
  void dispose() {
    _focusNode.dispose();
    _controller.dispose();
    super.dispose();
  }

  _onNationalNumberChanges() {
    if (_controller.text != value?.nsn) {
      final newPhoneNumber = widget.parser.parseWithIsoCode(
        isoCode,
        _controller.text,
      );
      didChange(newPhoneNumber);
    }
  }

  _onCountrySelected(Country country) {
    PhoneNumber newPhoneNumber =
        widget.parser.parseWithIsoCode(country.isoCode, value?.nsn ?? '');
    didChange(newPhoneNumber);
  }

  openCountrySelection() async {
    final selected = await widget.selectorNavigator.navigate(context);
    if (selected != null) {
      _onCountrySelected(selected);
    }
    _focusNode.requestFocus();
  }

  Widget builder() {
    // the idea here is to have a TextField with a prefix where the prefix
    // is the flag + dial code which is the same height as text so it's well
    // aligned with the typed text. It also does not push labels etc
    // around and keep the same form factor as TextFormField.
    //
    // Then we stack an InkWell on top of that to add the clickable part
    return Stack(
      children: [
        _textField(),
        if (_focusNode.hasFocus) _inkWellOverlay(),
      ],
    );
  }

  Widget _textField() {
    return TextFormField(
      focusNode: _focusNode,
      controller: _controller,
      onSaved: (p) => widget.onSaved!(value),
      style: widget.inputTextStyle,
      autofocus: widget.autofocus,
      autofillHints: widget.enabled && widget.withHint
          ? [AutofillHints.telephoneNumberNational]
          : null,
      enabled: widget.enabled,
      textDirection: TextDirection.ltr,
      keyboardType: TextInputType.phone,
      cursorColor: widget.cursorColor,
      inputFormatters: <TextInputFormatter>[
        FilteringTextInputFormatter.digitsOnly,
        FilteringTextInputFormatter.singleLineFormatter,
        // FilteringTextInputFormatter.allow(RegExp(r'^\d{0,30}$'))
      ],
      decoration: _getEffectiveDecoration(),
    );
  }

  Widget _inkWellOverlay() {
    return InkWell(
      onTap: () {},
      onTapDown: (_) => openCountrySelection(),
      // we make the country dial code
      // invisible but we still have to put it here
      // to have the correct width
      child: Opacity(
        opacity: 0,
        child: Padding(
          // outline border has padding on the left
          // so we need to make it a 12 bigger
          // and we add 16 horizontally to make it the whole height
          padding: isOutlineBorder
              ? const EdgeInsets.fromLTRB(12, 16, 0, 16)
              : const EdgeInsets.fromLTRB(0, 16, 0, 16),
          child: _getDialCodeChip(),
        ),
      ),
    );
  }

  InputDecoration _getEffectiveDecoration() {
    return widget.decoration.copyWith(
      errorText: getErrorText(),
      prefix: _getDialCodeChip(),
    );
  }

  Widget _getDialCodeChip() {
    return Padding(
      padding: const EdgeInsets.fromLTRB(8, 0, 8, 0),
      child: FlagDialCodeChip(
        country: Country(isoCode),
        showFlag: widget.showFlagInInput,
        textStyle: TextStyle(fontSize: 16),
        flagSize: 20,
      ),
    );
  }

  // which error text to show
  String? getErrorText() {
    if (!hasError) return null;
    return PhoneFieldLocalization.of(context)?.translate(errorText!) ??
        errorText;
  }
}
